%%Function to compute the convex conjugate of G using the DUAL problem, 
%in the form of linear programming. This function takes an input "p", the
%vector of conditional choice probabilities for a given state, and "S", the
%number of discretization of the error distribution. The output is the
%estimated vector of choice-specific per-period utilities at a given state. 

function u=Gs(p,S)

%D is the number of choices.
%p is a D-by-1 vector of choice probabilities. 
%S is the number of discretized points of the unobserved dist. 
%e is a S-by-D vector of S number of draws from the D-dimemsional 
%distribution of unobserved heterogeneity.

% Set seed for random stream
sst = RandStream('mt19937ar','Seed',4);
RandStream.setGlobalStream(sst);

e=mvnrnd([0,0],[0.5,0.5;0.5,1],S); %2d gaussian
D=numel(p);

%Aeq is a S+D by D*S matrix of equality constraints:
Aeq=zeros(S+D,D*S);
for d=1:1:D
    Aeq(d,(d-1)*S+1:d*S)=ones(1,S);
    Aeq(D+1:D+S,(d-1)*S+1:d*S)=eye(S);
end
Aeq=sparse(Aeq);

%The linear objective vector
f=zeros(D*S,1);
for d=1:1:D-1
    f((d-1)*S+1:d*S)=e(:,d);
end

f((D-1)*S+1:D*S)=zeros(S,1);
f=-f;

%The right-hand side vector for the linear constraints
w=(1/S)*ones(S,1);
rhs=[p';w];

%lower bound vector
lb=zeros(D*S,1);

%options = optimset('LargeScale', 'off', 'Simplex', 'on','MaxIter',10000);

options=optimset('Display', 'off');

[~,~,~,~,lambda]=linprog(f,[],[],Aeq,rhs,lb,[],[],options);

u=-lambda.eqlin(1:D)-mean(lambda.eqlin(d+1:end));

end

%% The codes below can be used in conjunction with the above to compute the size of the identified payoffs.

% Ceq=[p,zeros(1,S);zeros(1,d),w'];
% 
% 
% I=eye(d);
% 
%  
% [low1,~,exitflag]=linprog([I(:,1);zeros(S,1)],Aeq',f,Ceq,[fval;0],[],[],[],options);
% 
% 
% [low2,~,exitflag]=linprog([I(:,2);zeros(S,1)],Aeq',f,Ceq,[fval;0],[],[],[],options);
% 
% 
% [low3,~,exitflag]=linprog([I(:,3);zeros(S,1)],Aeq',f,Ceq,[fval;0],[],[],[],options);
% 
% 
% [up1,~,exitflag]=linprog(-[I(:,1);zeros(S,1)],Aeq',f,Ceq,[fval;0],[],[],[],options);
% 
% 
% [up2,~,exitflag]=linprog(-[I(:,2);zeros(S,1)],Aeq',f,Ceq,[fval;0],[],[],[],options);
% 
% 
% [up3,~,exitflag]=linprog(-[I(:,3);zeros(S,1)],Aeq',f,Ceq,[fval;0],[],[],[],options);
% 
% 
% low=[low1(1),low2(2),low3(3)];
% up=[up1(1),up2(2),up3(3)];
% 
% gap=up-low;



