%Monte-Carlo simulation. This script creates a matlab workspace for each MC
%design. Each workspace contains a matrix summarizing the goodness-of-fit
%statistics for each of the 100 replications, for that particular MC
%design. These numbers can be directly used to produce Table 1 and Table 3.

if matlabpool('size') == 0 % checking to see if pool is already open
      matlabpool open 8
end

tic

%% Initialize 
b=0.9; %discount factor
dim=30; %number of states
theta30 = 0.30; % parameters governing transition of states
theta31 = 0.35; 
theta32 = 0.25; 
theta33 = 0.1;


%% Require the true csvf (choice-specific value function) to simulate data, 
%therefore first run IBE(dim, b, rc, theta11, theta30,
%theta31, theta32, theta33) with rc=2, and theta11=0.5

[~,~,~,csvf,~] = IBE(dim, b, 2, 0.5, theta30, theta31, theta32, theta33);


%% Generate Simulated Data

%transition probability when y=1 
Pi1=zeros(dim,dim);
Pi1(:,1)=theta30*ones(dim,1);
Pi1(:,2)=theta31*ones(dim,1);
Pi1(:,3)=theta32*ones(dim,1);
Pi1(:,4)=theta33*ones(dim,1);


%transition probability when y=2 
Pi2=zeros(dim,dim);

Pi2(1:10,1)=theta30*ones(10,1);
Pi2(1:10,2)=theta31*ones(10,1);
Pi2(1:10,3)=theta32*ones(10,1);
Pi2(1:10,4)=theta33*ones(10,1);

for x=11:1:dim
    
    Pi2(x,x-10)=theta30;
    Pi2(x,x-9)=theta31;
    Pi2(x,x-8)=theta32;
    Pi2(x,x-7)=theta33;
    
end

%transition probability for y=3

Pi3=zeros(dim,dim);

for x=1:1:dim-3

    Pi3(x,x)=theta30;
    Pi3(x,x+1)=theta31;
    Pi3(x,x+2)=theta32;
    Pi3(x,x+3)=theta33;

end

Pi3(dim-2,dim-2)=(theta30)/(theta30+theta31+theta32);
Pi3(dim-2,dim-1)=(theta31)/(theta30+theta31+theta32);
Pi3(dim-2,dim)=(theta32)/(theta30+theta31+theta32);
Pi3(dim-1,dim-1)=(theta30)/(theta30+theta31);
Pi3(dim-1,dim)=(theta31)/(theta30+theta31);
Pi3(dim,dim)=1;


IT=100; %IT is the number of iterations


for TN=[500,500;1000,100;1000,1000]'
    
    N=TN(1);
    T=TN(2);

    slope=[];
  
parfor it=1:1:IT 
    
    
    Allbus=zeros(2,T,N);


    for n=1:N

             x=zeros(1,T);
             I=zeros(1,T);
             x(1)=unidrnd(dim); %set random starting state

        for t=1:1:T-1;


            e=mvnrnd([0,0],[0.5,0.5;0.5,1]);


            [~,I(t)]=max([csvf(x(t),1)+e(1),csvf(x(t),2)+e(2),csvf(x(t),3)],[],2);


            if I(t)==1  %choice #1

                u=unifrnd(0,1);  %simulate multinomial distribution
                if u <= theta30;
                    x(t+1)=1;
                elseif  (theta30 < u) && (u <= theta30+theta31)
                    x(t+1)=2;
                elseif (theta30+theta31 < u) && (u <= theta30+theta31+theta32)
                    x(t+1)=3;
                elseif theta30+theta31+theta32 < u 
                    x(t+1)=4;
                end

            elseif I(t)==2  %choice #2

                u=unifrnd(0,1);  %simulate multinomial distribution
                if u <= theta30;
                    x(t+1)=max(1,x(t)-10);
                elseif  (theta30 < u) && (u <= theta30+theta31)
                    x(t+1)=max(2,x(t)-9);
                elseif (theta30+theta31 < u) && (u <= theta30+theta31+theta32)
                    x(t+1)=max(3,x(t)-8);
                elseif theta30+theta31+theta32 < u 
                    x(t+1)=max(4,x(t)-7);
                end        


            elseif I(t)==3 

                u=unifrnd(0,1);
                if u <= theta30;
                    x(t+1)=x(t);
                elseif (theta30 < u) && (u <= theta30+theta31) && (x(t)<dim)
                    x(t+1)=x(t)+1;
                elseif (theta30 < u) && (u <= theta30+theta31) && (x(t)==dim)
                    x(t+1)=x(t);
                elseif (theta30+theta31 < u) && (u <= theta30+theta31+theta32) && (x(t)<dim-1)
                    x(t+1)=x(t)+2;
                elseif (theta30+theta31 < u) && (u <= theta30+theta31+theta32) && (x(t)>=dim-1)
                    x(t+1)=x(t);
                elseif theta30+theta31+theta32 < u && (x(t)<dim-2)
                    x(t+1)=x(t)+3;
                elseif theta30+theta31+theta32 < u && (x(t)>=dim-2)
                    x(t+1)=x(t);      
                end

            end     

        end


        Allbus(:,:,n)=[I;x];

    end


    i1=zeros(1,N);
    i2=zeros(1,N);
    i3=zeros(1,N);
    p=zeros(dim,4);


    for xs=1:1:dim

        for n=1:1:N

         index=find(Allbus(2,:,n)==xs);

         i1(n)=sum(Allbus(1,index,n)==1);
         i2(n)=sum(Allbus(1,index,n)==2);
         i3(n)=sum(Allbus(1,index,n)==3);

        end

        p(xs,1)=sum(i1)/(sum(i1)+sum(i2)+sum(i3));  %frequency of choice 1 in state x+1
        p(xs,2)=sum(i2)/(sum(i1)+sum(i2)+sum(i3));
        p(xs,3)=sum(i3)/(sum(i1)+sum(i2)+sum(i3));
        p(xs,4)=(sum(i1)+sum(i2)+sum(i3));

    end



    %% Estimate per-period payoffs using simulated data %%%%%%%%%%%%%%%%%%%
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
    %p(x,:) is the vector of observed choice probabilities in state x
 
    L=zeros(dim,3);

    %Core LP estimation

    for xs=1:1:dim

        L(xs,:)=Gs(p(xs,1:3),1000); %S=1000

    end

    %ex-ante value function

    Vx=(b*Pi3-eye(dim))\L(:,3);

    %vector of per-period utilities for y=1 and y=2

    u=zeros(dim,2);
    u(:,1)=L(:,1)+Vx-b*Pi1*Vx;
    u(:,2)=L(:,2)+Vx-b*Pi2*Vx;
    
    %restrict domain for u(:,1) to states with interior ccp
    
    us=find(p(1:10,1)>0,1,'last');
    uf=find(p(21:end,1)>0,1,'first')+20;
   

    %% Evaluate how well the estimated non-parametric utilities fit the true one.

    %true utility functions
    
    trueu1=-2*ones(30,1)+0.5*sqrt([1:1:30]');
    trueu2=-2*ones(30,1)+0.5*0.8*sqrt([1:1:30]');
    
    %R-squared
      
    rr1=1-(sum((u(us:uf,1)-trueu1(us:uf)).^2)/sum((u(us:uf,1)-mean(u(us:uf,1))).^2));
    rr2=1-(sum((u(:,2)-trueu2).^2)/sum((u(:,2)-mean(u(:,2))).^2));
    
    %RMSE
    
    rm1=sqrt(mean((u(us:uf,1)-trueu1(us:uf)).^2));
    rm2=sqrt(mean((u(:,2)-trueu2).^2));

    slope=[slope;[rr1,rr2,rm1,rm2]];


end


filename=['N',num2str(N),'T',num2str(T)];

save(filename,'slope')
    
%clearvars -except csvf slope theta30 theta31 theta32 theta33 dim b N T IT


end
    

toc

